=== modified file 'bzrlib/tests/workingtree_implementations/__init__.py'
--- bzrlib/tests/workingtree_implementations/__init__.py	2006-02-16 02:53:00 +0000
+++ bzrlib/tests/workingtree_implementations/__init__.py	2006-02-22 05:45:47 +0000
@@ -68,6 +68,7 @@
     result = TestSuite()
     test_workingtree_implementations = [
         'bzrlib.tests.workingtree_implementations.test_is_control_filename',
+        'bzrlib.tests.workingtree_implementations.test_pull',
         'bzrlib.tests.workingtree_implementations.test_workingtree',
         ]
     adapter = WorkingTreeTestProviderAdapter(

=== added file 'bzrlib/tests/workingtree_implementations/test_pull.py'
--- bzrlib/tests/workingtree_implementations/test_pull.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_pull.py	2006-02-22 05:45:47 +0000
@@ -0,0 +1,67 @@
+# (C) 2006 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+import os
+
+import bzrlib
+import bzrlib.branch
+from bzrlib.branch import Branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.bzrdir import BzrDir
+import bzrlib.errors as errors
+from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.osutils import basename
+from bzrlib.tests import TestSkipped
+from bzrlib.tests.workingtree_implementations import TestCaseWithWorkingTree
+from bzrlib.trace import mutter
+from bzrlib.transport import get_transport
+import bzrlib.workingtree as workingtree
+from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
+                                WorkingTree)
+
+
+class TestPull(TestCaseWithWorkingTree):
+
+    def get_pullable_trees(self):
+        self.build_tree(['from/', 'from/file', 'to/'])
+        tree = self.make_branch_and_tree('from')
+        tree.add('file')
+        tree.commit('foo', rev_id='A')
+        tree_b = self.make_branch_and_tree('to')
+        return tree, tree_b
+ 
+    def test_pull(self):
+        tree_a, tree_b = self.get_pullable_trees()
+        tree_b.pull(tree_a.branch)
+        self.failUnless(tree_b.branch.repository.has_revision('A'))
+        self.assertEqual('A', tree_b.last_revision())
+
+    def test_pull_overwrites(self):
+        tree_a, tree_b = self.get_pullable_trees()
+        tree_b.commit('foo', rev_id='B')
+        self.assertEqual(['B'], tree_b.branch.revision_history())
+        tree_b.pull(tree_a.branch, overwrite=True)
+        self.failUnless(tree_b.branch.repository.has_revision('A'))
+        self.failUnless(tree_b.branch.repository.has_revision('B'))
+        self.assertEqual('A', tree_b.last_revision())
+
+    def test_pull_merges_tree_content(self):
+        tree_a, tree_b = self.get_pullable_trees()
+        tree_b.pull(tree_a.branch)
+        self.assertFileEqual('contents of from/file\n', 'to/file')
+

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-02-16 02:53:00 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-02-22 05:45:47 +0000
@@ -80,29 +80,6 @@
         tree.unlock()
         self.assertEqual(None, tree.branch.peek_lock_mode())
  
-    def get_pullable_trees(self):
-        self.build_tree(['from/', 'from/file', 'to/'])
-        tree = self.make_branch_and_tree('from')
-        tree.add('file')
-        tree.commit('foo', rev_id='A')
-        tree_b = self.make_branch_and_tree('to')
-        return tree, tree_b
- 
-    def test_pull(self):
-        tree_a, tree_b = self.get_pullable_trees()
-        tree_b.pull(tree_a.branch)
-        self.failUnless(tree_b.branch.repository.has_revision('A'))
-        self.assertEqual('A', tree_b.last_revision())
-
-    def test_pull_overwrites(self):
-        tree_a, tree_b = self.get_pullable_trees()
-        tree_b.commit('foo', rev_id='B')
-        self.assertEqual(['B'], tree_b.branch.revision_history())
-        tree_b.pull(tree_a.branch, overwrite=True)
-        self.failUnless(tree_b.branch.repository.has_revision('A'))
-        self.failUnless(tree_b.branch.repository.has_revision('B'))
-        self.assertEqual('A', tree_b.last_revision())
-
     def test_revert(self):
         """Test selected-file revert"""
         tree = self.make_branch_and_tree('.')

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-02-21 21:06:53 +0000
+++ bzrlib/workingtree.py	2006-02-22 05:45:47 +0000
@@ -840,6 +840,7 @@
         source.lock_read()
         try:
             old_revision_history = self.branch.revision_history()
+            basis_tree = self.basis_tree()
             count = self.branch.pull(source, overwrite, stop_revision)
             new_revision_history = self.branch.revision_history()
             if new_revision_history != old_revision_history:
@@ -849,8 +850,8 @@
                     other_revision = None
                 repository = self.branch.repository
                 merge_inner(self.branch,
-                            self.basis_tree(), 
-                            repository.revision_tree(other_revision),
+                            self.branch.basis_tree(),
+                            basis_tree, 
                             this_tree=self, 
                             pb=bzrlib.ui.ui_factory.progress_bar())
                 self.set_last_revision(self.branch.last_revision())

